package com.mmbnetworks;

import java.util.Scanner;

import com.mmbnetworks.host.Host;
import com.mmbnetworks.host.HostInitializationException;
import com.mmbnetworks.serial.types.NodeId;
import com.mmbnetworks.serialdriver.SerialUtil;

public class Main {

    public static void main(String[] args) {

        if (args.length != 1) {
            System.out.println("Wrong number of arguments; expecting serial port as the only parameter");
            System.out.println("Example: HostApp /dev/tty.SLAB_USBtoUART");
            return;
        }

        // Create our host class
        final String devicePort = (String)args[0];
        Host host;
        try {
            host = new Host(devicePort);
        } catch (HostInitializationException e1) {
            e1.printStackTrace();
            return;
        }

        // use the main thread as a CLI
        try (Scanner scanner = new Scanner(System.in)) {

            while (true) {
                System.out.println("Enter a command: ");
                final String input = scanner.nextLine();

                if ("q".equalsIgnoreCase(input)) {
                    break;
                } else if ("m".equalsIgnoreCase(input)) {
                    host.moduleInfoRequest();
                } else if ("n".equalsIgnoreCase(input)) {
                    host.networkStatusRequest();
                } else if ("j".equalsIgnoreCase(input)) {
                    host.joinNetwork();
                } else if ("l".equalsIgnoreCase(input)) {
                    host.leaveNetwork();
                } else if ("c".equalsIgnoreCase(input)) {
                    System.out.println("Enter NodeID: ");
                    final String nodeId = scanner.nextLine();
                    NodeId n = new NodeId(SerialUtil.hexStringToMSBByteArray(nodeId));
                    System.out.println("Enter Endpoint: ");
                    final String endpoint = scanner.nextLine();
                    short e = Short.parseShort(endpoint);
                    System.out.println("Type ON for On Command, OFF for Off command, TOG for Toggle command: ");
                    final String option = scanner.nextLine();
                    if("on".equalsIgnoreCase(option)) {
                        host.sendOnCommand(n,e);
                    } else if("off".equalsIgnoreCase(option)){
                        host.sendOffCommandFrame(n,e);
                    } else if("tog".equalsIgnoreCase(option)){
                        host.sendToggleCommandFrame(n,e);
                    } else {
                        System.out.println("Invalid option selected!");
                    }
                } else if ("f".equalsIgnoreCase(input)) {
                    host.formNetwork();
                } else if ("p".equalsIgnoreCase(input)) {
                    host.permitJoin((short)30);
                } else if ("r".equalsIgnoreCase(input)) {
                    host.restoreFactoryDefaults();
                } else if ("serialUpgrade".equalsIgnoreCase(input)) {
                    System.out.println("Enter full file path to .mbl: ");
                    final String filePath = scanner.nextLine();
                    host.startSerialUpgrade(filePath);
                }
            }
        }

        // clean up
        host.disconnect();

        System.out.println ("Done!");
    }

}
