package com.mmbnetworks.clustercommands;

import com.mmbnetworks.serial.rha.zclmessages.RHASendZCLUnicast;
import com.mmbnetworks.serial.types.Bitmap8;
import com.mmbnetworks.serial.types.ClusterID;
import com.mmbnetworks.serial.types.NodeId;
import com.mmbnetworks.serial.types.OctetString;
import com.mmbnetworks.serial.types.UInt16;
import com.mmbnetworks.serial.types.UInt8;

public class ClusterCommands {

    private static final int CLUSTER_ON_OFF = 0x0006;
    private static final short CMD_OFF = 0x00;
    private static final short CMD_ON = 0x01;
    private static final short CMD_TOGGLE = 0x02;

    public static RHASendZCLUnicast getOnCommandFrame(final NodeId nodeId, final short endpointId) {

        return getClusterSpecificCommandFrame(nodeId, endpointId, CLUSTER_ON_OFF, CMD_ON, null, false);
    }
    public static RHASendZCLUnicast getOffCommandFrame(final NodeId nodeId, final short endpointId) {

        return getClusterSpecificCommandFrame(nodeId, endpointId, CLUSTER_ON_OFF, CMD_OFF, null, false);
    }
    public static RHASendZCLUnicast getToggleCommandFrame(final NodeId nodeId, final short endpointId) {

        return getClusterSpecificCommandFrame(nodeId, endpointId, CLUSTER_ON_OFF, CMD_TOGGLE, null, false);
    }

    private static RHASendZCLUnicast getClusterSpecificCommandFrame(final NodeId nodeId, final short endpointId, final int clusterId, final short commandId, final byte[] payload, final boolean requiresResponse) {
        final RHASendZCLUnicast unicast = new RHASendZCLUnicast();

        Bitmap8 frameControl = new Bitmap8();
        frameControl.setBit(0); // cluster-specific
        frameControl.clearBit(3); // send from client-to-server
        frameControl.setBit(4); // disable default response
        Bitmap8 sendOptions = new Bitmap8(); // no bits selected
        if (requiresResponse) {
            sendOptions.setBit(2); // wait for response
        }
        OctetString payloadOctetString = new OctetString();
        payloadOctetString.setValue((payload == null) ? new byte[0] : payload);

        unicast.setDestinationNodeID(nodeId);
        unicast.setDestinationEndpoint(new UInt8(endpointId));
        unicast.setClusterID(new ClusterID(clusterId));
        unicast.setLocalEndpoint(new UInt8((short) 0x01)); // we always set the endpoint of this host to 0x01
        unicast.setFrameControl(frameControl);
        unicast.setCommandID(new UInt8(commandId));
        unicast.setEncryptionLevel(new UInt8((short) 0x00));
        unicast.setOptions(sendOptions);
        unicast.setManufacturerCode(new UInt16(0x0000)); // not used
        unicast.setTransactionSequenceNumber(new UInt8((short) 0)); // not used
        unicast.setPayload(payloadOctetString);

        return unicast;
    }
}
