package com.mmbnetworks.serialdriver;

public class SerialUtil {

    public static String toHexString(int val, int digits) {
        String s = Integer.toHexString(val);
        int length = s.length();
        if (length < digits) {
            for (int i = 0; i < digits - length; i++) {
                s = "0" + s;
            }
        } else if (length > digits) {
            s = s.substring(length - digits, length);
        }

        return s.toUpperCase();
    }
    
    public static String toHexString(long val, int digits) {
        String s = Long.toHexString(val);
        int length = s.length();
        if (length < digits) {
            for (int i = 0; i < digits - length; i++) {
                s = "0" + s;
            }
        } else if (length > digits) {
            s = s.substring(length - digits, length);
        }
        return s.toUpperCase();
    }

    public static byte intToUnsignedByte(int i) {
        return (byte) (i & 0xFF);
    }

    public static int byteToInt(byte b) {
        return (int) (b & 0xFF);
    }

    public static short byteToShort(byte b) {
        return (short) (b & 0xFF);
    }

    public static int bytesToSignedInt(byte[] b) {
        int val = 0;
        int shift = 0;

        boolean isNegative = (b[b.length - 1] < 0);
        b[b.length - 1] &= ~(0x80);

        for (int i = 0; i < b.length; i++) {
            val += ((b[i] & 0xFF) << shift);
            shift += 8;
        }

        if (isNegative) {
            val = -(++val);

        }

        return val;
    }

    public static String bytesToHex(byte[] bytes) {
        final char[] hexArray = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
        char[] hexChars = new char[bytes.length * 3];
        int v;
        for (int j = 0; j < bytes.length; j++) {
            v = bytes[j] & 0xFF;
            hexChars[j * 3] = hexArray[v >>> 4];
            hexChars[j * 3 + 1] = hexArray[v & 0x0F];
            hexChars[j * 3 + 2] = ' ';
        }
        return new String(hexChars);
    }

    public static String bytesToMSBHexNoSpaces(byte[] bytes) {
        final char[] hexArray = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
        char[] hexChars = new char[bytes.length * 2];
        int v;
        for (int j = 0; j < bytes.length; j++) {
            v = bytes[j] & 0xFF;
            hexChars[((bytes.length - j) * 2) - 2] = hexArray[v >>> 4];
            hexChars[((bytes.length - j) * 2) - 1] = hexArray[v & 0x0F];
        }
        return new String(hexChars);
    }
    
    public static String toHexString(byte[] bytes) {
        final String[] hexArray = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F"};
        String result = "";
        for(byte b: bytes) {
            result += hexArray[(b & 0xF0) >> 4];
            result += hexArray[b & 0x0F];
        }
        return result;
    }

    public static byte[] hexStringToMSBByteArray(String s) {
        if (validHexString(s)) {
            int len = s.length();
            if (len % 2 != 0) {
                len += 1;
                s = "0" + s;
            }
            byte[] data = new byte[len / 2];
            for (int i = 0; i < len; i += 2) {
                data[i / 2] = (byte) ((Character.digit(s.charAt(len - 2 - i), 16) << 4)
                        + Character.digit(s.charAt(len - 1 - i), 16));
            }
            return data;
        } else {
            throw new IllegalArgumentException("Not Valid Hex");
        }

    }

    public static byte[] hexStringToMSBByteArray(String s, int numOfBytes) {
        if (validHexString(s)) {
            int len = s.length();
            int adjust = numOfBytes * 2 - len;
            if (adjust > 0) {
                for (int i = 0; i < adjust; i++) {
                    s = "0" + s;
                }
                len = s.length();
            } else {
                //do nothing as the string is a good enough length
            }
            byte[] data = new byte[len / 2];
            for (int i = 0; i < len; i += 2) {
                data[i / 2] = (byte) ((Character.digit(s.charAt(len - 2 - i), 16) << 4)
                        + Character.digit(s.charAt(len - 1 - i), 16));
            }
            return data;
        } else {
            throw new IllegalArgumentException("Not Valid Hex");
        }
    }

    public static byte[] hexStringToLSBByteArray(String s) {
        if (validHexString(s)) {
            int len = s.length();
            if (len % 2 != 0) {
                len += 1;
                s = "0" + s;
            }
            byte[] data = new byte[len / 2];
            for (int i = 0; i < len; i += 2) {
                data[i / 2] = (byte) ((Character.digit(s.charAt(i), 16) << 4)
                        + Character.digit(s.charAt(i + 1), 16));
            }
            return data;
        } else {
            throw new IllegalArgumentException("Not Valid Hex");
        }
    }

    public static byte hexStringToByte(String s) {
        try {
            if (s.length() % 2 != 0) {
                s = "0" + s;
            }
            int val = Integer.parseInt(s, 16);
            return (byte) (val & 0xFF);
        } catch (NumberFormatException nfe) {

        }
        return 0;
    }

    public static short hexStringToShort(String s) {
        try {
            if (s.length() % 2 != 0) {
                s = "0" + s;
            }
            int val = Integer.parseInt(s, 16);
            return (short) (val & 0xFF);
        } catch (NumberFormatException nfe) {

        }
        return 0;
    }

    public static String hexToAscii(String hex) {
        String hexString = hex.replaceAll("\\s+", "");
        StringBuilder sb = new StringBuilder();
        StringBuilder temp = new StringBuilder();

        for (int i = 0; i < hexString.length() - 1; i += 2) {
            String output = hexString.substring(i, (i + 2));
            int decimal = Integer.parseInt(output, 16);
            sb.append((char) decimal);

            temp.append(decimal);
        }
        return sb.toString();
    }

    public static boolean validHexString(String s) {
        return s.matches("[\\dA-Fa-f0-9]+") || s.isEmpty();
    }
}
